@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2024 OARC, Inc.
+ * Copyright (c) 2016-2025 OARC, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,21 @@
 #endif
 #endif
 
+/* Macros for printing timeval */
+#include <inttypes.h>
+#if _TIME_BITS == 64
+#define PRI_tv_sec PRId64
+#define PRI_tv_usec PRId64
+#else
+#ifdef __OpenBSD__
+#define PRI_tv_sec PRId64
+#define PRI_tv_usec "ld"
+#else
+#define PRI_tv_sec "ld"
+#define PRI_tv_usec "ld"
+#endif
+#endif
+
 /*
  * setup MY_BPFTIMEVAL as the timeval structure that bpf packets
  * will be assoicated with packets from libpcap
@@ -140,6 +155,9 @@ typedef void (*tcpstate_reset_t)(void* tcpstate, const char* msg);
 #define DNSCAP_EXT_SET_IADDR 5
 typedef void (*set_iaddr_t)(iaddr* from, iaddr* to);
 
+#define DNSCAP_EXT_SET_OUTPUT_PKT 6
+typedef void (*set_output_pkt_t)(u_char* pkt, const unsigned olen);
+
 /*
  * Flags
  */
