@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2024 OARC, Inc.
+ * Copyright (c) 2016-2025 OARC, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -192,25 +192,35 @@ void layer_pkt(u_char* user, const pcap_thread_packet_t* packet, const u_char* p
                 break;
         }
 
-        t = (time_t)firstpkt->pkthdr.ts.tv_sec;
+        t = firstpkt->pkthdr.ts.tv_sec;
         gmtime_r(&t, &tm);
         strftime(when, sizeof(when), "%Y-%m-%d %T", &tm);
 
+        if (packet->have_iphdr && !options.defrag_ipv4) {
+            if (len > packet->iphdr.ip_len) {
+                len = packet->iphdr.ip_len;
+            }
+        } else if (packet->have_ip6hdr && !options.defrag_ipv6) {
+            if (len > 40 + packet->ip6hdr.ip6_plen) {
+                len = 40 + packet->ip6hdr.ip6_plen;
+            }
+        }
+
         if (vlan != MAX_VLAN) {
-            snprintf(descr, sizeof(descr), "[%lu] %s.%06lu [#%ld %s (vlan %u) %u] \\\n",
-                (u_long)len,
+            snprintf(descr, sizeof(descr), "[%zu] %s.%06" PRI_tv_usec " [#%zd %s (vlan %u) %u] \\\n",
+                len,
                 when,
-                (u_long)firstpkt->pkthdr.ts.tv_usec,
-                (long)msgcount,
+                firstpkt->pkthdr.ts.tv_usec,
+                msgcount,
                 mypcap->name ? mypcap->name : "\"some interface\"",
                 vlan,
                 vlan);
         } else {
-            snprintf(descr, sizeof(descr), "[%lu] %s.%06lu [#%ld %s %u] \\\n",
-                (u_long)len,
+            snprintf(descr, sizeof(descr), "[%zu] %s.%06" PRI_tv_usec " [#%zd %s %u] \\\n",
+                len,
                 when,
-                (u_long)firstpkt->pkthdr.ts.tv_usec,
-                (long)msgcount,
+                firstpkt->pkthdr.ts.tv_usec,
+                msgcount,
                 mypcap->name ? mypcap->name : "\"some interface\"",
                 vlan);
         }
@@ -229,7 +239,7 @@ void layer_pkt(u_char* user, const pcap_thread_packet_t* packet, const u_char* p
 
     network_pkt2(descr, firstpkt->pkthdr.ts, packet, payload, length);
 
-    if (limit_packets != 0U && msgcount == limit_packets) {
+    if (limit_packets != 0U && msgcount >= limit_packets) {
         if (preso)
             goto breakloop;
         if (dumper_opened == dump_state && dumper_close(firstpkt->pkthdr.ts))
@@ -356,6 +366,16 @@ void dl_pkt(u_char* user, const struct pcap_pkthdr* hdr, const u_char* pkt, cons
         len -= 16;
         break;
     }
+#endif
+#ifdef DLT_LINUX_SLL2
+    case DLT_LINUX_SLL2: {
+        if (len < 20)
+            return;
+        etype = _need16(&pkt[0]);
+        pkt += 20;
+        len -= 20;
+        break;
+    }
 #endif
     default:
         return;
@@ -406,7 +426,7 @@ void dl_pkt(u_char* user, const struct pcap_pkthdr* hdr, const u_char* pkt, cons
         struct tm   tm;
         time_t      t;
 
-        t = (time_t)hdr->ts.tv_sec;
+        t = hdr->ts.tv_sec;
         gmtime_r(&t, &tm);
         strftime(when, sizeof when, "%Y-%m-%d %T", &tm);
         if (vlan != MAX_VLAN) {
@@ -417,8 +437,8 @@ void dl_pkt(u_char* user, const struct pcap_pkthdr* hdr, const u_char* pkt, cons
         } else {
             viap = "\"some interface\"";
         }
-        snprintf(descr, sizeof(descr), "[%lu] %s.%06lu [#%ld %s %u] \\\n",
-            (u_long)len, when, (u_long)hdr->ts.tv_usec, (long)msgcount, viap, vlan);
+        snprintf(descr, sizeof(descr), "[%zu] %s.%06" PRI_tv_usec " [#%zu %s %u] \\\n",
+            len, when, hdr->ts.tv_usec, msgcount, viap, vlan);
     } else {
         descr[0] = '\0';
     }
@@ -436,7 +456,7 @@ void dl_pkt(u_char* user, const struct pcap_pkthdr* hdr, const u_char* pkt, cons
 
     network_pkt(descr, hdr->ts, pf, pkt, len);
 
-    if (limit_packets != 0U && msgcount == limit_packets) {
+    if (limit_packets != 0U && msgcount >= limit_packets) {
         if (preso)
             goto breakloop;
         if (dumper_opened == dump_state && dumper_close(hdr->ts))
@@ -705,11 +725,11 @@ void network_pkt2(const char* descr, my_bpftimeval ts, const pcap_thread_packet_
 
         tcpstate = tcpstate_find(from, to, sport, dport, ts.tv_sec);
         if (dumptrace >= 3) {
-            fprintf(stderr, "%s: tcp pkt: %lu.%06lu [%4lu] %15s -> ",
+            fprintf(stderr, "%s: tcp pkt: %" PRI_tv_sec ".%06" PRI_tv_usec " [%4zu] %15s -> ",
                 ProgramName,
-                (u_long)ts.tv_sec,
-                (u_long)ts.tv_usec,
-                (u_long)len,
+                ts.tv_sec,
+                ts.tv_usec,
+                len,
                 ia_str(from));
             fprintf(stderr, "%15s; ", ia_str(to));
 
@@ -753,7 +773,7 @@ void network_pkt2(const char* descr, my_bpftimeval ts, const pcap_thread_packet_
                     fprintf(stderr, "warning: recycling state for "
                         "duplicate tcp stream after only %ld "
                         "seconds idle\n",
-                        (u_long)(ts.tv_sec - tcpstate->last_use));
+                        ts.tv_sec - tcpstate->last_use);
                     */
                 }
             } else {
@@ -1089,6 +1109,7 @@ void network_pkt(const char* descr, my_bpftimeval ts, unsigned pf,
     struct ip*      ip;
     size_t          len, dnslen = 0;
     HEADER          dns;
+    char            descr_[512];
 
     if (dumptrace >= 4)
         fprintf(stderr, "processing %s packet: len=%zu\n", (pf == PF_INET ? "IPv4" : (pf == PF_INET6 ? "IPv6" : "unknown")), olen);
@@ -1118,8 +1139,16 @@ void network_pkt(const char* descr, my_bpftimeval ts, unsigned pf,
         to.af = AF_INET;
         memcpy(&to.u.a4, &ip->ip_dst, sizeof(struct in_addr));
         offset = ip->ip_hl << 2;
-        if (len > ntohs(ip->ip_len)) /* small IP packets have L2 padding */
+        if (len > ntohs(ip->ip_len)) {
+            /* IP packets have L2 padding */
+            olen -= len - ntohs(ip->ip_len); // remove ether frame padding
+            if (preso) {
+                // rewrite descr
+                snprintf(descr_, sizeof(descr_), "[%zu]%s", olen, strchr(descr, ' '));
+                descr = descr_;
+            }
             len = ntohs(ip->ip_len);
+        }
         if (len <= (size_t)offset)
             goto network_pkt_end;
         pkt += offset;
@@ -1151,6 +1180,16 @@ void network_pkt(const char* descr, my_bpftimeval ts, unsigned pf,
         offset      = sizeof(struct ip6_hdr);
         payload_len = ntohs(ipv6->ip6_plen);
 
+        if (len > sizeof(*ipv6) + payload_len) {
+            /* IP packets have L2 padding */
+            olen -= len - sizeof(*ipv6) - payload_len; // remove ether frame padding
+            if (preso) {
+                // rewrite descr
+                snprintf(descr_, sizeof(descr_), "[%zu]%s", olen, strchr(descr, ' '));
+                descr = descr_;
+            }
+        }
+
         memset(&from, 0, sizeof from);
         from.af = AF_INET6;
         memcpy(&from.u.a6, &ipv6->ip6_src, sizeof(struct in6_addr));
@@ -1288,8 +1327,8 @@ void network_pkt(const char* descr, my_bpftimeval ts, unsigned pf,
 
         tcpstate = tcpstate_find(from, to, sport, dport, ts.tv_sec);
         if (dumptrace >= 3) {
-            fprintf(stderr, "%s: tcp pkt: %lu.%06lu [%4lu] ", ProgramName,
-                (u_long)ts.tv_sec, (u_long)ts.tv_usec, (u_long)len);
+            fprintf(stderr, "%s: tcp pkt: %" PRI_tv_sec ".%06" PRI_tv_usec " [%4zu] ", ProgramName,
+                ts.tv_sec, ts.tv_usec, len);
             fprintf(stderr, "%15s -> ", ia_str(from));
             fprintf(stderr, "%15s; ", ia_str(to));
             if (tcpstate)
@@ -1325,7 +1364,7 @@ void network_pkt(const char* descr, my_bpftimeval ts, unsigned pf,
                 fprintf(stderr, "warning: recycling state for "
                     "duplicate tcp stream after only %ld "
                     "seconds idle\n",
-                    (u_long)(ts.tv_sec - tcpstate->last_use));
+                    ts.tv_sec - tcpstate->last_use);
             }
 #endif
             } else {
